angular.module('btapp.caches').factory('blcache', function($rootScope,$cordovaToast) {
	/*
	 * 注意：所有缓存数据在原有基础上增加一个cache：true的字段，
	 *     标记为是缓存数据，将离线数据存储到数据库时只处理cache:true的数据
	 */
	var bluetoothCache = {
		// 存储当前用户所属系统的所有设备信息（字符串，需要的时候转换成JSON对象，为了考虑空间，考虑是否压缩数据）
		devices : window.localStorage.devices ? window.localStorage.devices : '[]',
		// 存储当前用户所属系统的所有锁具信息（字符串，需要的时候转换成JSON对象）
		locksets : window.localStorage.locksets ? window.localStorage.locksets : '[]',
		// 存储当前用户和设备的权限关系信息（字符串，需要的时候转换成JSON对象）
		deviceUserRelations : window.localStorage.lockPerm ? window.localStorage.lockPerm : '[]',
		// 存储开锁记录
		hisDatas : window.localStorage.hisData ? window.localStorage.hisData : '[]',
		
		// 根据锁具id获取锁具信息
		getPubLocksetByLockId : function(lockId) {
			var locksets = JSON.parse(window.localStorage.locksets);
			var lockset = null;
	    	for(var i = 0, len = locksets.length; i < len; i++) {
	    		if (locksets[i].lockId == lockId) {
	    			lockset = locksets[i];
	    			break;
	    		}
	    	}
	    	return lockset;
		},
		// 根据锁具id获取设备信息
		getPubDeviceByLockId : function(lockId) {
			var devices = JSON.parse(window.localStorage.devices);
	    	var device = null;
	    	for(var i = 0, len = devices.length; i < len; i++) {
    			if (devices[i].lockId == lockId) {
    				device = devices[i];
    				break;
    			}
    		}
	    	return device;
		},
		// 根据用户Id获取设备和用户的关系列表
		getDeviceUserRelationByUserId : function(userId) {
			var relations = [];
			var deviceUserRelations = JSON.parse(window.localStorage.lockPerm);
			for(var i = 0, len = deviceUserRelations.length; i < len; i++) {
				if (deviceUserRelations[i]['userId'] == userId) {
					//处理权限关系 设备为-100的设备
					var record = deviceUserRelations[i];
					if(record.deviceId == -100){
						record.device = {
							'deviceId':-100,
							'deviceName':'全部设备',
							'location':'不限地址'
						}
					}
					relations.push(record);
				}
			}
			return relations;
		},
		// 挂锁，把设备表中的状态根据depend进行更新，并更新锁具表中的安装状态及时间
		installPubDeviceAndPubLockset : function(device) {
			this.updatePubDevice(device);
			this.updatePubLockset(device.lockId, device.deviceName, 1, device.time);
			this.updateDeviceUserRelation(device);
			
		},
		// 回收锁具，把设备表中的状态根据depend进行更新，并更新锁具表中的安装状态及时间
		recoverPubDeviceAndPubLockset : function(device) {
			var lockId = device.lockId;
			if (device.depend == 0) {
				device.lockId = 0;
			} else {
				device.deviceName = device.lockId;
			}
			device.installer = 0;
			device.longitude = 0;
			device.latitude = 0;
			this.updatePubDevice(device);
			this.updatePubLockset(lockId, ('回收锁' + device.lockId), 0, device.time);
			this.updateDeviceUserRelation(device);
		},
		// 根据depend情况进行更新
		updatePubDevice : function(device) {
			var devices = JSON.parse(window.localStorage.devices);
	    	for(var i = 0, len = devices.length; i < len; i++) {
    			var tempDevice = devices[i];
    			if (tempDevice.deviceId == device.deviceId) {
    				device.cache = true; // 增加cache标志位
    				devices[i] = device;
    			}
    		}
	    	this.refreshDevicesCache(devices);
		},
		// 根据设备更新锁具信息，保持和设备信息匹配
		updatePubLockset : function(lockId, description, installStatus, time) {
			var locksets = JSON.parse(window.localStorage.locksets);
	    	for(var i = 0, len = locksets.length; i < len; i++) {
	    		var tempLockset = locksets[i];
	    		if (tempLockset.lockId == lockId) {
	    			locksets[i]['lockName'] = description;
	    			locksets[i]['installStatus'] = installStatus;
	    			locksets[i]['time'] = time;
	    			locksets[i]['cache'] = true;// 增加cache标志位
	    		}
	    	}
	    	this.refreshLocksetsCache(locksets);
		},
		// 更新设备关系表中的device的信息
		updateDeviceUserRelation : function(device) {
			var deviceUserRelations = JSON.parse(window.localStorage.lockPerm);
			for(var i = 0, len = deviceUserRelations.length; i < len; i++) {
				if (deviceUserRelations[i]['deviceId'] == -100) {
					continue;
				}
				var oldDevice = deviceUserRelations[i]['device'];
				if (oldDevice == null) {
					continue;
				}
				if (oldDevice.deviceId == device.deviceId) {
					deviceUserRelations[i]['device'] = device;
					break;
				}
			}
			this.refreshDeviceUserRelationCache(deviceUserRelations);
		},
		
		// 存储开锁记录
		insertPubHisData : function(pubHisData) {
			var hisDatas = JSON.parse(window.localStorage.hisData);
			hisDatas.push(pubHisData);
			this.hisDatas = JSON.stringify(hisDatas);
			window.localStorage.hisData = this.hisDatas;
		},
		clearPubHisDataCache : function() {
			this.hisDatas = '[]';
			window.localStorage.hisData = '[]';
		},
		refreshDevicesCache : function(newDevices) {
			this.devices = JSON.stringify(newDevices);
			window.localStorage.devices = this.devices;
		},
		refreshLocksetsCache : function(newLocksets) {
			this.locksets = JSON.stringify(newLocksets);
			window.localStorage.locksets = this.locksets;
		},
		refreshDeviceUserRelationCache : function(deviceUserRelations) {
			this.deviceUserRelations = JSON.stringify(deviceUserRelations);
			window.localStorage.lockPerm = this.deviceUserRelations;
		},
		refreshAllCache : function(res) {
			this.refreshDevicesCache(res.devices);
			this.refreshLocksetsCache(res.locksets);
			this.refreshDeviceUserRelationCache(res.permission);
		},
		// 查询cache == true的设备信息，需要同步到服务器
		getPubDeviceByCache : function() {
			var devices = JSON.parse(window.localStorage.devices);
			var cacheDevices = [];
			for(var i = 0, len = devices.length; i < len; i++) {
    			var tempDevice = devices[i]; //depend=1时会有deviceId = -100 的关系，此时device = null  ---by leaf
    			if (tempDevice != null && tempDevice.cache) {
    				delete tempDevice.cache;
    				cacheDevices.push(tempDevice);
    			}
    		}
			return cacheDevices;
		},
		// 查询cache == true的锁具信息，需要同步到服务器
		getPubLocksetByCache : function() {
			var locksets = JSON.parse(window.localStorage.locksets);
			var cacheLocksets = [];
	    	for(var i = 0, len = locksets.length; i < len; i++) {
	    		var tempLockset = locksets[i];
	    		if (tempLockset.cache) {
	    			delete tempLockset.cache;
	    			cacheLocksets.push(tempLockset);
	    		}
	    	}
	    	return cacheLocksets;
		},
		// 查询开锁记录，需要同步到服务器
		getPubHisDataByCache : function() {
			var datas = JSON.parse(window.localStorage.hisData);
			return datas;
		},
		getSecondByDate : function(date) {
			return date.getTime() / 1000;
		},
		getNowSecondTime : function() {
			return this.getSecondByDate(new Date());
		},
		getLastTime : function() {
	    	var date = new Date();
	    	return date.getTime();
	    },
		isDevTimeSync : function() {
			var updateTime = window.localStorage.upateTime;
	    	var nowTime = this.getLastTime();
	    	if (updateTime - 0 > 0) {
	    		if ((nowTime - updateTime) < -60000) { // 60s
	        		return false;
	        	}
	    	}
	    	return true;
	    }
	};
	return bluetoothCache;
});